package org.greenstone.gsdl3.gs3build.doctypes;

import java.io.PrintWriter;

import java.sql.*;

import org.greenstone.gsdl3.gs3build.util.XMLTools;
import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;
import org.greenstone.gsdl3.gs3build.database.*;

public class DocumentSQLWriter
{
  public DocumentSQLWriter()
  {
  }

  public static boolean touchDocument(DocumentID docID, GS3SQLConnection connection, long touchTime)
  { GS3SQLUpdate update = new GS3SQLUpdate("document");
    update.setWhere(new GS3SQLWhere(new GS3SQLWhereItem("DocID", "=", docID.toString())));
    update.addDate("IndexedDate", new java.sql.Timestamp(touchTime));
    connection.execute(update.toString());
    System.out.println(update.toString());
    return true;
  }

  /**
   * Just write the document to the database, assuming that it is not already there
   */
  public boolean writeDocument(DocumentInterface document, GS3SQLConnection connection)
  {		
    // TODO: output document in METS format, with Greenstone-specific metadata
    // placed in a wrapper inside the descriptive metadata block

    // put the document into the database
    try {
      if (document.getID() != null)
      { //tag = XMLTools.addAttribute(tag, "OBJID", document.getID().toString());
	GS3SQLSelect select = new GS3SQLSelect("document");
	select.addField("*");
	select.setWhere(new GS3SQLWhere(new GS3SQLWhereItem("DocID", "=", document.getID().toString())));
	connection.execute(select.toString());
	
	ResultSet results = connection.getResultSet();

	if (results == null ||
	    !results.first())
	{ GS3SQLInsert insert = new GS3SQLInsert("document");
	  insert.addValue("DocID", document.getID().toString());
	  insert.addValue("DocType", document.getDocumentType());

	  insert.addDate("AccessionDate", new java.sql.Timestamp(document.getAccessionDate()));
	  insert.addDate("IndexedDate", new java.sql.Timestamp(document.getLastIndexedDate()));
	  insert.addDate("ModifiedDate", new java.sql.Timestamp(document.getModifiedDatestamp()));

	  connection.execute(insert.toString());
	}
	else {
	  /* redundant code - not used... */
	  GS3SQLUpdate update = new GS3SQLUpdate("document");
	  update.setWhere(new GS3SQLWhere(new GS3SQLWhereItem("DocID", "=", document.getID().toString())));
	  connection.execute(update.toString());
	}
      }
    } catch (Exception ex) {
	System.out.println(ex);
    }
    //    output.println(tag);
    
    // Write the structural metadata:
    //   if in "Create METS" mode:
    //     take all the documents as being in the one group, of the "physical"
    //     document; particular document plugins may extend this behaviour,
    //     which would suggest placing it in the AbstractDocument or similar
    //     class?!
    if (document.getDocumentMetadata() == null) {
      System.out.println("Null metadata document");
    }
    document.getDocumentMetadata().writeSQL(document, connection);

    document.getDocumentFiles().writeSQL(document, connection);

    document.getDocumentStructure().writeSQL(document, connection);

    // Write the body information
    // TODO: only output the document if there is some
    // content/formatting change between the original and
    // what greenstone actually does; if the original is
    // good, then place a marker saying "get the document
    // "from the original file"
    return true;
  }
}
